"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var LasError = /** @class */ (function () {
    function LasError(message) {
        this.name = 'LasError';
        this.message = message;
    }
    return LasError;
}());
var ColumnError = /** @class */ (function () {
    function ColumnError(column) {
        this.name = 'ColumnError';
        this.message = "Column " + column + " doesn't exist in the file";
    }
    return ColumnError;
}());
var PathError = /** @class */ (function () {
    function PathError() {
        this.name = 'PathError';
        this.message = 'Path is invalid';
    }
    return PathError;
}());
var CsvError = /** @class */ (function () {
    function CsvError() {
        this.name = 'CsvError';
        this.message = "Couldn't convert file to CSV";
    }
    return CsvError;
}());
var PropertyError = /** @class */ (function () {
    function PropertyError(property) {
        this.name = 'PropertyError';
        this.message = "Property " + property + " doesn't exist";
    }
    return PropertyError;
}());
// eslint-disable-next-line @typescript-eslint/no-unused-vars
var Las = /** @class */ (function () {
    /**
     * Creates an instance of Las.
     * @param {string} path - Absolute path to las file
     * @memberof Las
     */
    function Las(path) {
        this.path = path;
        this.blobString = this.initialize();
    }
    Las.chunk = function (arr, size) {
        var overall = [];
        var index = 0;
        while (index < arr.length) {
            overall.push(arr.slice(index, index + size));
            index += size;
        }
        return overall;
    };
    Las.removeComment = function (str) {
        return str
            .trim()
            .split('\n')
            .map(function (val) { return val.trimLeft(); })
            .filter(function (f) { return !(f.charAt(0) === '#'); })
            .join('\n');
    };
    Las.convertToValue = function (s) {
        return Boolean(+s) || /^0|0$/.test(s) ? +s : s;
    };
    /**
     * Returns a column in a las file
     * @param {string} column - name of column
     * @returns {(Promise<Array<string| number>>)}
     * @memberof Las
     */
    Las.prototype.column = function (column) {
        return __awaiter(this, void 0, void 0, function () {
            var hds, sB, index;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.header()];
                    case 1:
                        hds = _a.sent();
                        return [4 /*yield*/, this.data()];
                    case 2:
                        sB = _a.sent();
                        index = hds.findIndex(function (item) { return item.toLowerCase() === column.toLowerCase(); });
                        if (index < 0) {
                            throw new ColumnError(column);
                        }
                        return [2 /*return*/, sB.map(function (c) { return c[index]; })];
                }
            });
        });
    };
    /**
     * Returns a column in a las file stripped off null values
     * @param {string} column - name of column
     * @returns {(Promise<Array<string| number>>)}
     * @memberof Las
     */
    Las.prototype.columnStripped = function (column) {
        return __awaiter(this, void 0, void 0, function () {
            var hds, sB, index;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.header()];
                    case 1:
                        hds = _a.sent();
                        return [4 /*yield*/, this.dataStripped()];
                    case 2:
                        sB = _a.sent();
                        index = hds.findIndex(function (item) { return item.toLowerCase() === column.toLowerCase(); });
                        if (index >= 0) {
                            return [2 /*return*/, sB.map(function (c) { return c[index]; })];
                        }
                        else {
                            throw new ColumnError(column);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns a csv File object in browser | writes csv file to current working driectory in Node
     * @param {string} filename
     * @returns {(Promise<File | void>)}
     * @memberof Las
     */
    Las.prototype.toCsv = function (filename) {
        if (filename === void 0) { filename = 'file'; }
        return __awaiter(this, void 0, void 0, function () {
            var headers, data, rHd, rData, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 3, , 4]);
                        return [4 /*yield*/, this.header()];
                    case 1:
                        headers = _a.sent();
                        return [4 /*yield*/, this.data()];
                    case 2:
                        data = _a.sent();
                        rHd = headers.join(',') + '\n';
                        rData = data.map(function (d) { return d.join(','); }).join('\n');
                        return [2 /*return*/, new File([rHd + rData], filename + ".csv")];
                    case 3:
                        error_1 = _a.sent();
                        throw new CsvError();
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns a csv File object in browser and writes csv file to current working driectory in Node of data stripped of null values
     * @param {string} filename
     * @returns {(Promise<File | void>)}
     * @memberof Las
     */
    Las.prototype.toCsvStripped = function (filename) {
        if (filename === void 0) { filename = 'file'; }
        return __awaiter(this, void 0, void 0, function () {
            var headers, data, rHd, rData, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 3, , 4]);
                        return [4 /*yield*/, this.header()];
                    case 1:
                        headers = _a.sent();
                        return [4 /*yield*/, this.dataStripped()];
                    case 2:
                        data = _a.sent();
                        rHd = headers.join(',') + '\n';
                        rData = data.map(function (d) { return d.join(','); }).join('\n');
                        return [2 /*return*/, new File([rHd + rData], filename + ".csv")];
                    case 3:
                        error_2 = _a.sent();
                        throw new CsvError();
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns the number of rows in a .las file
     * @returns number
     * @memberof Las
     */
    Las.prototype.rowCount = function () {
        return __awaiter(this, void 0, void 0, function () {
            var l;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.data()];
                    case 1:
                        l = _a.sent();
                        return [2 /*return*/, l.length];
                }
            });
        });
    };
    /**
     * Returns the number of columns in a .las file
     * @returns number
     * @memberof Las
     */
    Las.prototype.columnCount = function () {
        return __awaiter(this, void 0, void 0, function () {
            var l;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.header()];
                    case 1:
                        l = _a.sent();
                        return [2 /*return*/, l.length];
                }
            });
        });
    };
    /**
     * Returns a two-dimensional array of data in the log
     * @returns {(Promise<Array<Array<string | number>>>)}
     * @memberof Las
     */
    Las.prototype.data = function () {
        return __awaiter(this, void 0, void 0, function () {
            var s, hds, totalheadersLength, sB, con;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.blobString];
                    case 1:
                        s = _a.sent();
                        return [4 /*yield*/, this.header()];
                    case 2:
                        hds = _a.sent();
                        totalheadersLength = hds.length;
                        sB = s
                            .split(/~A(?:\w*\s*)*\n/)[1]
                            .trim()
                            .split(/\s+/)
                            .map(function (m) { return Las.convertToValue(m.trim()); });
                        if (sB.length < 0) {
                            throw new LasError('No data/~A section in the file');
                        }
                        con = Las.chunk(sB, totalheadersLength);
                        return [2 /*return*/, con];
                }
            });
        });
    };
    /**
     * Returns a two-dimensional array of data in the log with all rows containing null values stripped off
     * @returns {(Promise<Array<Array<string | number>>>)}
     * @memberof Las
     */
    Las.prototype.dataStripped = function () {
        return __awaiter(this, void 0, void 0, function () {
            var s, hds, well, nullValue, totalheadersLength, sB, con, filtered;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.blobString];
                    case 1:
                        s = _a.sent();
                        return [4 /*yield*/, this.header()];
                    case 2:
                        hds = _a.sent();
                        return [4 /*yield*/, this.property('well')];
                    case 3:
                        well = _a.sent();
                        nullValue = well.NULL.value;
                        totalheadersLength = hds.length;
                        sB = s
                            .split(/~A(?:\w*\s*)*\n/)[1]
                            .trim()
                            .split(/\s+/)
                            .map(function (m) { return Las.convertToValue(m.trim()); });
                        if (sB.length < 0) {
                            throw new LasError('No data/~A section in the file');
                        }
                        con = Las.chunk(sB, totalheadersLength);
                        filtered = con.filter(function (f) { return !f.some(function (x) { return x === +nullValue; }); });
                        return [2 /*return*/, filtered];
                }
            });
        });
    };
    /**
     * Returns the version number of the las file
     * @returns {Promise<number>}
     * @memberof Las
     */
    Las.prototype.version = function () {
        return __awaiter(this, void 0, void 0, function () {
            var v;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.metadata()];
                    case 1:
                        v = _a.sent();
                        return [2 /*return*/, v[0]];
                }
            });
        });
    };
    /**
     * Returns true if the las file is of wrapped variant and false otherwise
     * @returns {Promise<boolean>}
     * @memberof Las
     */
    Las.prototype.wrap = function () {
        return __awaiter(this, void 0, void 0, function () {
            var v;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.metadata()];
                    case 1:
                        v = _a.sent();
                        return [2 /*return*/, v[1]];
                }
            });
        });
    };
    /**
     * Returns an extra info about the well stored in ~others section
     * @returns {Promise<string>}
     * @memberof Las
     */
    Las.prototype.other = function () {
        return __awaiter(this, void 0, void 0, function () {
            var s, som, str, some;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.blobString];
                    case 1:
                        s = _a.sent();
                        som = s.split(/~O(?:\w*\s*)*\n\s*/i)[1];
                        str = '';
                        if (som) {
                            some = som
                                .split('~')[0]
                                .replace(/\n\s*/g, ' ')
                                .trim();
                            str = Las.removeComment(some);
                        }
                        if (str.length <= 0) {
                            throw new LasError('No ~other section');
                        }
                        return [2 /*return*/, str];
                }
            });
        });
    };
    /**
     * Returns an array of strings of the logs header/title
     * @returns {Promise<string[]>}
     * @memberof Las
     */
    Las.prototype.header = function () {
        return __awaiter(this, void 0, void 0, function () {
            var s, sth, uncommentedSth;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.blobString];
                    case 1:
                        s = _a.sent();
                        sth = s.split(/~C(?:\w*\s*)*\n\s*/)[1].split('~')[0];
                        uncommentedSth = Las.removeComment(sth).trim();
                        if (uncommentedSth.length < 0) {
                            throw new LasError('There is no header in the file');
                        }
                        return [2 /*return*/, uncommentedSth.split('\n').map(function (m) { return m.trim().split(/\s+|[.]/)[0]; })];
                }
            });
        });
    };
    /**
     * Returns an object, each well header and description as a key-value pair
     * @returns {Promise<{[key:string]: string}>}
     * @memberof Las
     */
    Las.prototype.headerAndDescr = function () {
        return __awaiter(this, void 0, void 0, function () {
            var cur, hd, descr, obj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.property('curve')];
                    case 1:
                        cur = (_a.sent());
                        hd = Object.keys(cur);
                        descr = Object.values(cur).map(function (c, i) { return (c.description === 'none' ? hd[i] : c.description); });
                        obj = {};
                        hd.map(function (_, i) { return (obj[hd[i]] = descr[i]); });
                        if (Object.keys(obj).length < 0) {
                            throw new LasError('Poorly formatted ~curve section in the file');
                        }
                        return [2 /*return*/, obj];
                }
            });
        });
    };
    /**
     * Returns details of  well parameters.
     * @returns {Promise<WellProps>}
     * @memberof Las
     */
    Las.prototype.wellParams = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.property('well')];
            });
        });
    };
    /**
     * Returns details of  curve parameters.
     * @returns {Promise<WellProps>}
     * @memberof Las
     */
    Las.prototype.curveParams = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.property('curve')];
            });
        });
    };
    /**
     * Returns details of  parameters of the well.
     * @returns {Promise<WellProps>}
     * @memberof Las
     */
    Las.prototype.logParams = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.property('param')];
            });
        });
    };
    Las.prototype.metadata = function () {
        return __awaiter(this, void 0, void 0, function () {
            var str, sB, sw, refined, res, wrap;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.blobString];
                    case 1:
                        str = _a.sent();
                        sB = str
                            .trim()
                            .split(/~V(?:\w*\s*)*\n\s*/)[1]
                            .split(/~/)[0];
                        sw = Las.removeComment(sB);
                        refined = sw
                            .split('\n')
                            .map(function (m) { return m.split(/\s{2,}|\s*:/).slice(0, 2); })
                            .filter(function (f) { return Boolean(f); });
                        res = refined.map(function (r) { return r[1]; });
                        wrap = res[1].toLowerCase() === 'yes' ? true : false;
                        if ([+res[0], wrap].length < 0) {
                            throw new LasError("Couldn't get metadata");
                        }
                        return [2 /*return*/, [+res[0], wrap]];
                }
            });
        });
    };
    Las.prototype.property = function (p) {
        return __awaiter(this, void 0, void 0, function () {
            var regDict, regExp, str, substr, sw, res, s_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        regDict = {
                            curve: '~C(?:\\w*\\s*)*\\n\\s*',
                            param: '~P(?:\\w*\\s*)*\\n\\s*',
                            well: '~W(?:\\w*\\s*)*\\n\\s*'
                        };
                        regExp = new RegExp(regDict[p], 'i');
                        return [4 /*yield*/, this.blobString];
                    case 1:
                        str = _a.sent();
                        substr = str.split(regExp);
                        sw = '';
                        if (substr.length > 1) {
                            res = substr[1].split(/~/)[0];
                            sw = Las.removeComment(res);
                        }
                        if (sw.length > 0) {
                            s_1 = {};
                            sw.split('\n').map(function (c) {
                                var obj = c.replace(/\s*[.]\s+/, '   none   ');
                                var title = obj.split(/[.]|\s+/)[0];
                                var unit = obj
                                    .trim()
                                    .split(/^\w+\s*[.]*s*/)[1]
                                    .split(/\s+/)[0];
                                var description = Boolean(obj.split(/[:]/)[1].trim()) ? obj.split(/[:]/)[1].trim() : 'none';
                                var third = obj.split(/[:]/)[0].split(/\s{2,}\w*\s{2,}/);
                                var value = third.length > 2 && !Boolean(third[third.length - 1]) ? third[third.length - 2] : third[third.length - 1];
                                value = value.length > 0 ? value.trim() : value;
                                s_1[title] = { unit: unit, value: value, description: description };
                            });
                            return [2 /*return*/, s_1];
                        }
                        else {
                            throw new PropertyError(p);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    Las.prototype.initialize = function () {
        return __awaiter(this, void 0, void 0, function () {
            var val, text, error_3;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.path instanceof File)) return [3 /*break*/, 1];
                        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                var reader = new FileReader();
                                reader.onerror = function () {
                                    reject(new PathError());
                                };
                                reader.onload = function () {
                                    resolve(reader.result);
                                };
                                reader.readAsText(_this.path);
                            })];
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, fetch(this.path)];
                    case 2:
                        val = _a.sent();
                        return [4 /*yield*/, val.text()];
                    case 3:
                        text = _a.sent();
                        if (text.includes('404: Not Found')) {
                            throw new PathError();
                        }
                        return [2 /*return*/, text];
                    case 4:
                        error_3 = _a.sent();
                        throw new PathError();
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    return Las;
}());
